﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;

namespace Eq2FullscreenWindowizer
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool EnumWindows(EnumWindowsProc lpEnumFunc, IntPtr lParam);

        private delegate bool EnumWindowsProc(IntPtr hWnd, IntPtr lParam);

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern int GetWindowTextLength(IntPtr hWnd);

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static extern int GetWindowText(IntPtr hWnd, StringBuilder lpString, int nMaxCount);

        private enum WindowLongIndex : int
        {
            GWL_EXSTYLE = -20,
            GWL_STYLE = -16,
            GWL_WNDPROC = -4,
            GWL_HINSTANCE = -6,
            GWL_HWNDPARENT = -8,
            GWL_ID = -12,
            GWL_USERDATA = -21,
            DWL_DLGPROC = 4,
            DWL_MSGRESULT = 0,
            DWL_USER = 8
        }

        private enum WindowStyles : uint
        {
            WS_MAXIMIZEBOX = 0x10000, /* x */
            WS_MINIMIZEBOX = 0x20000, /* x */
            WS_THICKFRAME = 0x40000, /* x */
            WS_SYSMENU = 0x80000, /* x */
            WS_HSCROLL = 0x100000,
            WS_VSCROLL = 0x200000,
            WS_DLGFRAME = 0x400000, /* x */
            WS_BORDER = 0x800000, /* x */
            WS_MAXIMIZE = 0x1000000,
            WS_CLIPCHILDREN = 0x2000000,
            WS_CLIPSIBLINGS = 0x4000000,
            WS_DISABLED = 0x8000000,
            WS_VISIBLE = 0x10000000,
            WS_MINIMIZE = 0x20000000,
            WS_CHILD = 0x40000000,
            WS_POPUP = 0x80000000
        }

        private enum WindowStylesEx : uint
        {
            WS_EX_DLGMODALFRAME = 0x0001,
            WS_EX_NOPARENTNOTIFY = 0x0004,
            WS_EX_TOPMOST = 0x0008,
            WS_EX_ACCEPTFILES = 0x0010,
            WS_EX_TRANSPARENT = 0x0020,
            WS_EX_MDICHILD = 0x0040,
            WS_EX_TOOLWINDOW = 0x0080,
            WS_EX_WINDOWEDGE = 0x0100,
            WS_EX_CLIENTEDGE = 0x0200,
            WS_EX_CONTEXTHELP = 0x0400,
            WS_EX_RIGHT = 0x1000,
            WS_EX_RTLREADING = 0x2000,
            WS_EX_LEFTSCROLLBAR = 0x4000,
            WS_EX_CONTROLPARENT = 0x10000,
            WS_EX_STATICEDGE = 0x20000,
            WS_EX_APPWINDOW = 0x40000,
            WS_EX_LAYERED = 0x00080000,
            WS_EX_NOINHERITLAYOUT = 0x00100000, // Disable inheritence of mirroring by children
            WS_EX_LAYOUTRTL = 0x00400000, // Right to left mirroring
            WS_EX_COMPOSITED = 0x02000000,
            WS_EX_NOACTIVATE = 0x08000000
        }

        [Flags]
        private enum SetWindowPosFlags : uint
        {
            SWP_NOSIZE = 0x0001,
            SWP_NOMOVE = 0x0002,
            SWP_NOZORDER = 0x0004,
            SWP_NOREDRAW = 0x0008,
            SWP_NOACTIVATE = 0x0010,
            SWP_FRAMECHANGED = 0x0020,  /* The frame changed: send WM_NCCALCSIZE */
            SWP_SHOWWINDOW = 0x0040,
            SWP_HIDEWINDOW = 0x0080,
            SWP_NOCOPYBITS = 0x0100,
            SWP_NOOWNERZORDER = 0x0200,  /* Don't do owner Z ordering */
            SWP_NOSENDCHANGING = 0x0400  /* Don't send WM_WINDOWPOSCHANGING */
        }

        [DllImport("user32.dll")]
        static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int X, int Y, int cx, int cy, SetWindowPosFlags uFlags);

        [DllImport("user32.dll", SetLastError = true)]
        static extern int GetWindowLong(IntPtr hWnd, WindowLongIndex nIndex);

        private static WindowStyles GetWindowStyles(IntPtr hWnd)
        {
            return (WindowStyles)GetWindowLong(hWnd, WindowLongIndex.GWL_STYLE);
        }

        private static WindowStylesEx GetWindowStylesEx(IntPtr hWnd)
        {
            return (WindowStylesEx)GetWindowLong(hWnd, WindowLongIndex.GWL_EXSTYLE);
        }

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, WindowLongIndex nIndex, int dwNewLong);

        private static void SetWindowStyles(IntPtr hWnd, WindowStyles styles)
        {
            SetWindowLong(hWnd, WindowLongIndex.GWL_STYLE, (int)styles);
        }

        private static void SetWindowStylesEx(IntPtr hWnd, WindowStylesEx stylesEx)
        {
            SetWindowLong(hWnd, WindowLongIndex.GWL_EXSTYLE, (int)stylesEx);
        }

        private class CbScreen
        {
            public CbScreen(int idx, Screen scr)
            {
                m_idx = idx;
                m_screen = scr;
            }

            private readonly int m_idx;
            private readonly Screen m_screen;

            public Screen Screen
            {
                get { return m_screen; }
            }

            public override string ToString()
            {
                return m_idx.ToString() + ": " + m_screen.DeviceName;
            }
        }

        private const string RegKey = "HKEY_CURRENT_USER\\SOFTWARE\\Eq2FullScreenWindowizer";
        private const string RegValue = "ScreenIndex";

        private void Form1_Load(object sender, EventArgs e)
        {
            int idx = 0;
            foreach (Screen scr in Screen.AllScreens)
            {
                idx++;
                comboBox1.Items.Add(new CbScreen(idx, scr));
            }

            int defScreenIdx;
            try
            {
                defScreenIdx =
                    Convert.ToInt32(
                        Microsoft.Win32.Registry.GetValue(RegKey, RegValue, 0));
            }
            catch
            {
                defScreenIdx = 0;
            }

            comboBox1.SelectedIndex = defScreenIdx;
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                Microsoft.Win32.Registry.SetValue(RegKey, RegValue, comboBox1.SelectedIndex);
            }
            catch { }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            EnumWindows(EnumWindowsCallbackSet, IntPtr.Zero);
        }

        private bool EnumWindowsCallbackSet(IntPtr hWnd, IntPtr lParam)
        {
            int winTextLength = GetWindowTextLength(hWnd);
            StringBuilder sb = new StringBuilder(winTextLength + 1);
            GetWindowText(hWnd, sb, sb.Capacity + 1);
            string winTitle = sb.ToString();
            if (winTitle.Contains("EverQuest II") && winTitle.Contains("SOEBuild"))
            {
                //listBox1.Items.Add(hWnd.ToString("x") + " \"" + winTitle + "\"");
                ThreadPool.QueueUserWorkItem(delegate
                {
                    Screen scr = ((CbScreen)comboBox1.SelectedItem).Screen;
                    Rectangle r = scr.Bounds;

                    WindowStyles ws = GetWindowStyles(hWnd);
                    ws ^= WindowStyles.WS_MAXIMIZEBOX | WindowStyles.WS_MINIMIZEBOX | WindowStyles.WS_THICKFRAME |
                        WindowStyles.WS_SYSMENU | WindowStyles.WS_DLGFRAME | WindowStyles.WS_BORDER;

                    SetWindowStyles(hWnd, ws);
                    SetWindowPos(hWnd, IntPtr.Zero, r.X, r.Y, r.Width, r.Height,
                        /*SetWindowPosFlags.SWP_NOMOVE | SetWindowPosFlags.SWP_NOSIZE |*/ SetWindowPosFlags.SWP_NOZORDER |
                        SetWindowPosFlags.SWP_FRAMECHANGED);
                });
                return false;
            }

            return true;
        }

    }
}
